@@ -35,8 +35,8 @@
 #define inline
 #endif
 
-#include <py3c.h>
-#include <py3c/capsulethunk.h>
+#include <Python.h>
+
 #include <stdlib.h>
 #include <math.h>
 #include <lz4.h>
@@ -76,14 +76,6 @@ typedef unsigned __int64 uint64_t;
 #endif /* __CHAR_BIT__ */
 #endif /* _WIN32 && _MSC_VER */
 
-#ifndef Py_UNUSED /* This is already defined for Python 3.4 onwards */
-#ifdef __GNUC__
-#define Py_UNUSED(name) _unused_ ## name __attribute__((unused))
-#else
-#define Py_UNUSED(name) _unused_ ## name
-#endif
-#endif
-
 #define LZ4_VERSION_NUMBER_1_9_0 10900
 
 static const char * stream_context_capsule_name = "_stream.LZ4S_ctx";
@@ -790,7 +782,8 @@ _create_context (PyObject * Py_UNUSED (self), PyObject * args, PyObject * kwds)
   Py_buffer dict = { NULL, NULL, };
 
   int status = 0;
-  int total_size = 0;
+  unsigned int total_size = 0;
+  uint32_t store_max_size;
 
   static char * argnames[] = {
     "strategy",
@@ -805,19 +798,13 @@ _create_context (PyObject * Py_UNUSED (self), PyObject * args, PyObject * kwds)
     NULL
   };
 
-#if IS_PY3
-#define _ARG_FMT "ssI|sIIpIz*"
-#else
-#define _ARG_FMT "ssI|sIIiIz*"
-#endif
-  if (!PyArg_ParseTupleAndKeywords (args, kwds, _ARG_FMT, argnames,
+  if (!PyArg_ParseTupleAndKeywords (args, kwds, "ssI|sIIpIz*", argnames,
                                     &strategy_name, &direction, &buffer_size,
                                     &mode, &acceleration, &compression_level, &return_bytearray,
                                     &store_comp_size, &dict))
     {
       goto abort_now;
     }
-#undef _ARG_FMT
 
   /* Sanity checks on arguments */
   if (dict.len > INT_MAX)
@@ -831,10 +818,11 @@ _create_context (PyObject * Py_UNUSED (self), PyObject * args, PyObject * kwds)
    *   https://github.com/lz4/lz4/blob/dev/lib/lz4.h#L161
    *
    * So, restrict the block length bitwise to 32 (signed 32 bit integer). */
-  if ((store_comp_size != 1) && (store_comp_size != 2) && (store_comp_size != 4))
+  if ((store_comp_size != 0) && (store_comp_size != 1) &&
+      (store_comp_size != 2) && (store_comp_size != 4))
     {
       PyErr_Format (PyExc_ValueError,
-                    "Invalid store_comp_size, valid values: 1, 2 or 4");
+                    "Invalid store_comp_size, valid values: 0, 1, 2 or 4");
       goto abort_now;
     }
 
@@ -902,13 +890,24 @@ _create_context (PyObject * Py_UNUSED (self), PyObject * args, PyObject * kwds)
       goto abort_now;
     }
 
-  /* Initialize the output buffer */
+  /* Initialize the output buffer
+   *
+   * In out-of-band block size case, use a best-effort strategy for scaling
+   * buffers.
+   */
+  if (store_comp_size == 0)
+    {
+      store_max_size = _GET_MAX_UINT32(4);
+    }
+  else
+    {
+      store_max_size = _GET_MAX_UINT32(store_comp_size);
+    }
+
   if (context->config.direction == COMPRESS)
     {
-      uint32_t store_max_size = _GET_MAX_UINT32(store_comp_size);
-
       context->output.len = get_compress_bound (buffer_size);
-      total_size = context->output.len + context->config.store_comp_size;
+      total_size = context->output.len + store_comp_size;
 
       if (context->output.len == 0)
         {
@@ -933,7 +932,6 @@ _create_context (PyObject * Py_UNUSED (self), PyObject * args, PyObject * kwds)
     }
   else /* context->config.direction == DECOMPRESS */
     {
-      uint32_t store_max_size = _GET_MAX_UINT32(store_comp_size);
       if (store_max_size > LZ4_MAX_INPUT_SIZE)
         {
           store_max_size = LZ4_MAX_INPUT_SIZE;
@@ -1074,17 +1072,10 @@ _compress_bound (PyObject * Py_UNUSED (self), PyObject * args)
   /* Positional arguments: input_size
    * Keyword arguments   : none
    */
-#if IS_PY3
-#define _ARG_FMT "OI"
-#else
-#define _ARG_FMT "OI"
-#endif
-  if (!PyArg_ParseTuple (args, _ARG_FMT,
-                         &input_size))
+  if (!PyArg_ParseTuple (args, "OI", &input_size))
     {
       goto exit_now;
     }
-#undef _ARG_FMT
 
   py_dest = PyLong_FromUnsignedLong (get_compress_bound (input_size));
 
@@ -1106,17 +1097,10 @@ _input_bound (PyObject * Py_UNUSED (self), PyObject * args)
   /* Positional arguments: compress_max_size
    * Keyword arguments   : none
    */
-#if IS_PY3
-#define _ARG_FMT "I"
-#else
-#define _ARG_FMT "I"
-#endif
-  if (!PyArg_ParseTuple (args, _ARG_FMT,
-                         &compress_max_size))
+  if (!PyArg_ParseTuple (args, "I", &compress_max_size))
     {
       goto exit_now;
     }
-#undef _ARG_FMT
 
   py_dest = PyLong_FromUnsignedLong (get_input_bound (compress_max_size));
 
@@ -1171,17 +1155,10 @@ _compress (PyObject * Py_UNUSED (self), PyObject * args)
   /* Positional arguments: capsule_context, source
    * Keyword arguments   : none
    */
-#if IS_PY3
-#define _ARG_FMT "Oy*"
-#else
-#define _ARG_FMT "Os*"
-#endif
-  if (!PyArg_ParseTuple (args, _ARG_FMT,
-                         &py_context, &source))
+  if (!PyArg_ParseTuple (args, "Oy*", &py_context, &source))
     {
       goto exit_now;
     }
-#undef _ARG_FMT
 
   context = _PyCapsule_get_context (py_context);
   if ((context == NULL) || (context->lz4_state.context == NULL))
@@ -1269,17 +1246,10 @@ _get_block (PyObject * Py_UNUSED (self), PyObject * args)
    * Keyword arguments   : none
    */
 
-#if IS_PY3
-#define _ARG_FMT "Oy*"
-#else
-#define _ARG_FMT "Os*"
-#endif
-  if (!PyArg_ParseTuple (args, _ARG_FMT,
-                         &py_context, &source))
+  if (!PyArg_ParseTuple (args, "Oy*", &py_context, &source))
     {
       goto exit_now;
     }
-#undef _ARG_FMT
 
   context = _PyCapsule_get_context (py_context);
   if ((context == NULL) || (context->lz4_state.context == NULL))
@@ -1295,6 +1265,13 @@ _get_block (PyObject * Py_UNUSED (self), PyObject * args)
       goto exit_now;
     }
 
+  if (context->config.store_comp_size == 0)
+    {
+      PyErr_Format (LZ4StreamError,
+                    "LZ4 context is configured for storing block size out-of-band");
+      goto exit_now;
+    }
+
   if (source.len < context->config.store_comp_size)
     {
       PyErr_Format (LZ4StreamError,
@@ -1343,22 +1320,16 @@ _decompress (PyObject * Py_UNUSED (self), PyObject * args)
   PyObject * py_context = NULL;
   PyObject * py_dest = NULL;
   int output_size = 0;
+  uint32_t source_size_max = 0;
   Py_buffer source = { NULL, NULL, };
 
   /* Positional arguments: capsule_context, source
    * Keyword arguments   : none
    */
-#if IS_PY3
-#define _ARG_FMT "Oy*"
-#else
-#define _ARG_FMT "Os*"
-#endif
-  if (!PyArg_ParseTuple (args, _ARG_FMT,
-                         &py_context, &source))
+  if (!PyArg_ParseTuple (args, "Oy*", &py_context, &source))
     {
       goto exit_now;
     }
-#undef _ARG_FMT
 
   context = _PyCapsule_get_context (py_context);
   if ((context == NULL) || (context->lz4_state.context == NULL))
@@ -1367,7 +1338,19 @@ _decompress (PyObject * Py_UNUSED (self), PyObject * args)
       goto exit_now;
     }
 
-  if (source.len > _GET_MAX_UINT32(context->config.store_comp_size))
+  /* In out-of-band block size case, use a best-effort strategy for scaling
+   * buffers.
+   */
+  if (context->config.store_comp_size == 0)
+    {
+      source_size_max = _GET_MAX_UINT32(4);
+    }
+  else
+    {
+      source_size_max = _GET_MAX_UINT32(context->config.store_comp_size);
+    }
+
+  if (source.len > source_size_max)
     {
       PyErr_Format (PyExc_OverflowError,
                     "Source length (%ld) too large for LZ4 store_comp_size (%d) value",
@@ -1649,16 +1632,10 @@ static PyModuleDef moduledef = {
 };
 
 
-MODULE_INIT_FUNC (_stream)
+PyMODINIT_FUNC
+PyInit__stream(void)
 {
-  PyObject * module = NULL;
-
-#if IS_PY3
-  module = PyModule_Create (&moduledef);
-#else
-  (void) moduledef; /* unused in python2 */
-  module = Py_InitModule("_stream", module_methods);
-#endif
+  PyObject * module = PyModule_Create (&moduledef);
 
   if (module == NULL)
     {
